/*==============================================================================

                                    O  F  E  L  I

                           Object  Finite  Element  Library

  ==============================================================================

   Copyright (C) 1998 - 2014 Rachid Touzani

   This file is part of OFELI.

   OFELI is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   OFELI is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with OFELI. If not, see <http://www.gnu.org/licenses/>.

  ==============================================================================

                   Various functions to determine matrix graph

  ==============================================================================*/


#ifndef __GRAPH_OF_MATRIX_H
#define __GRAPH_OF_MATRIX_H

#include <math.h>
#include <float.h>
#include <stdlib.h>
#include <vector>
#include <algorithm>

#include "mesh/Mesh.h"
#include "util/util.h"

namespace OFELI {

#ifndef DOXYGEN_SHOULD_SKIP_THIS
typedef std::pair<size_t,size_t> RC;

using std::vector;
using std::sort;

template<class T_>
class Vect;

size_t SimpleSkyline(const Mesh&           m,
                           vector<size_t>& ch);
size_t NodeSkyline(const Mesh&           m,
                         vector<size_t>& ch);
size_t NodeSkyline(const Mesh&           m,
                         vector<size_t>& ch,
                         size_t          dof1,
                         size_t         dof2);
size_t NodeSkyline(const Mesh& m,
                         vector<size_t>& ch,
                         size_t          dof);
size_t SideSkyline(const Mesh& m,
                         vector<size_t>& ch);
size_t SideSkyline(const Mesh& m,
                         vector<size_t>& ch,
                         size_t          dof1,
                         size_t          dof2);
size_t SideSkyline(const Mesh&           m,
                         vector<size_t>& ch,
                         size_t          dof);
size_t ElementSkyline(const Mesh&           m,
                            vector<size_t>& ch);
size_t ElementSkyline(const Mesh&           m,
                            vector<size_t>& ch,
                            size_t          dof);
size_t SimpleGraph(const Mesh&           m,
                         vector<long>&   xadj,
                         vector<size_t>& adjncy);
int NodeGraph(const Mesh& m,
                    int*  nnz,
                    int*  col_ind);
size_t NodeGraph(const Mesh&           m,
                       vector<size_t>& row_ptr,
                       vector<size_t>& col_ind);
size_t NodeGraph(const Mesh&           m,
                       size_t          dof1,
                       size_t          dof2,
                       vector<size_t>& row_ptr,
                       vector<size_t>& col_ind);
size_t SideGraph(const Mesh&           m,
                       vector<size_t>& row_ptr,
                       vector<size_t>& col_ind);
size_t SideGraph(const Mesh&           m,
                       size_t          dof1,
                       size_t          dof2,
                       vector<size_t>& row_ptr,
                       vector<size_t>& col_ind);
size_t ElementGraph(const Mesh&            m,
                          vector<size_t>& row_ptr,
                          vector<size_t>& col_ind);
size_t SideNodeGraph(const Mesh&           m,
                           vector<size_t>& row_ptr,
                           vector<size_t>& col_ind);
size_t NodeSideGraph(const Mesh&           m,
                           vector<size_t>& row_ptr,
                           vector<size_t>& col_ind);
size_t NodeGraphScal(const Mesh& m,
                           vector<size_t>& row_ptr,
                           vector<size_t>& col_ind);
size_t NodeGraphScal(const Mesh&           m,
                           size_t          dof,
                           size_t          nb_eq,
                           vector<size_t>& row_ptr,
                           vector<size_t>& col_ind);
size_t XGraph(const Mesh& m,
                    vector<size_t> &row_ptr,
                    vector<size_t> &col_ind);
size_t DG0Graph(const Mesh& m,
                      Vect<std::pair<size_t,size_t> >& I);
size_t DGGraph(const Mesh& m, Vect<std::pair<size_t,size_t> >& I);
void StoreGraph(      size_t size,
                const std::vector<RC>& p,
                      vector<size_t>& row_ptr,
                      vector<size_t>& col_ind);
#endif /* DOXYGEN_SHOULD_SKIP_THIS */

} /* namespace OFELI */

#endif
